@@ -1,4 +1,4 @@
-#!/usr/bin/perl -w
+#!@PERL@ -w
 #
 # usage: 
 #    check_flexlm.pl license_file
@@ -26,7 +26,6 @@
 # initial version: 9-10-99 Ernst-Dieter Martin edmt@infineon.com
 #
 # License: GPL
-# $Id: check_flexlm.pl,v 1.6 2003/02/04 06:16:16 sghosh Exp $
 #
 # lmstat output patches from Steve Rigler/Cliff Rice 13-Apr-2002
 # srigler@marathonoil.com,cerice@marathonoil.com
@@ -36,7 +35,9 @@
 use strict;
 use Getopt::Long;
 use vars qw($opt_V $opt_h $opt_F $opt_t $verbose $PROGNAME);
-use lib utils.pm;
+use FindBin;
+use lib "$FindBin::Bin";
+use lib '@libexecdir@';
 use utils qw(%ERRORS &print_revision &support &usage);
 
 $PROGNAME="check_flexlm";
@@ -44,7 +45,7 @@ $PROGNAME="check_flexlm";
 sub print_help ();
 sub print_usage ();
 
-$ENV{'PATH'}='';
+$ENV{'PATH'}='@TRUSTED_PATH@';
 $ENV{'BASH_ENV'}=''; 
 $ENV{'ENV'}='';
 
@@ -57,7 +58,7 @@ GetOptions
 	 "t=i" => \$opt_t, "timeout=i"  => \$opt_t);
 
 if ($opt_V) {
-	print_revision($PROGNAME,'$Revision: 1.6 $');
+	print_revision($PROGNAME,'@NP_VERSION@');
 	exit $ERRORS{'OK'};
 }
 
@@ -217,7 +218,7 @@ sub print_usage () {
 }
 
 sub print_help () {
-	print_revision($PROGNAME,'$Revision: 1.6 $');
+	print_revision($PROGNAME,'@NP_VERSION@');
 	print "Copyright (c) 2000 Ernst-Dieter Martin/Karl DeBisschop
 
 Check available flexlm license managers
